import pygame
from . import *

FORCE_FULL_SCREEN = pygame.FULLSCREEN if is_android else 0
FORCE_SCALED = pygame.SCALED if is_android else 0

LABEL_ALIGN_LEFT = 0
LABEL_ALIGN_CENTER = 1
LABEL_ALIGN_RIGHT = 2

WIN_BUTTON_ALIGN_LEFT = WIN_BUTTON_ALIGN_TOP = 0
WIN_BUTTON_ALIGN_CENTER = 1
WIN_BUTTON_ALIGN_RIGHT = WIN_BUTTON_ALIGN_BOTTOM = 2
